Package: pencal
Title: Penalized Regression Calibration for the Dynamic Prediction of
        Survival
Version: 2.3.1
Authors@R: c(
    person("Mirko", "Signorelli", , "msignorelli.rpackages@gmail.com", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0002-8102-3356")),
    person("Pietro", "Spitali", role = "ctb"),
    person("Roula", "Tsonaka", role = "ctb"),
    person("Barbara", "Vreede", role = "ctb")
  )
Description: Computes penalized regression calibration (PRC), a
    statistical method for the dynamic prediction of survival when many
    longitudinal predictors are available. See Signorelli (2024)
    <doi:10.32614/RJ-2024-014> and Signorelli et al. (2021)
    <doi:10.1002/sim.9178> for details.
License: GPL (>= 3)
URL: https://mirkosignorelli.github.io/r
Depends: R (>= 4.2.0)
VignetteBuilder: knitr
biocViews:
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: doParallel, dplyr, foreach, glmnet, lcmm, magic, MASS, Matrix,
        methods, nlme, purrr, riskRegression, stats, survcomp,
        survival, survivalROC
Suggests: knitr, ptmixed, rmarkdown, survminer
NeedsCompilation: no
Packaged: 2026-04-09 11:10:18 UTC; ms
Author: Mirko Signorelli [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-8102-3356>),
  Pietro Spitali [ctb],
  Roula Tsonaka [ctb],
  Barbara Vreede [ctb]
Maintainer: Mirko Signorelli <msignorelli.rpackages@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-09 12:30:02 UTC
Built: R 4.6.0; ; 2026-04-25 13:59:06 UTC; unix
