# Generated by roxygen2: do not edit by hand

S3method(print,prclmm)
S3method(print,prcmlpmm)
S3method(print,sprclmm)
S3method(print,sprcmlpmm)
S3method(summary,lmmfit)
S3method(summary,mlpmmfit)
S3method(summary,prclmm)
S3method(summary,prcmlpmm)
S3method(summary,ranefs)
export(fit_lmms)
export(fit_mlpmms)
export(fit_prclmm)
export(fit_prcmlpmm)
export(pencox)
export(performance_pencox)
export(performance_prc)
export(simulate_prclmm_data)
export(simulate_prcmlpmm_data)
export(simulate_t_weibull)
export(summarize_lmms)
export(summarize_mlpmms)
export(survplot_prc)
export(survpred_prclmm)
export(survpred_prcmlpmm)
import(doParallel)
import(foreach)
import(glmnet)
import(lcmm)
import(nlme)
import(stats)
import(survcomp)
import(survival)
import(survivalROC)
importFrom(Matrix,bdiag)
importFrom(dplyr,arrange)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(lcmm,multlcmm)
importFrom(methods,is)
