% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary_prclmm.R
\name{summary.prclmm}
\alias{summary.prclmm}
\title{Summary method for PRC LMM model fits}
\usage{
\method{summary}{prclmm}(object, ...)
}
\arguments{
\item{object}{an object of class \code{prclmm}}

\item{...}{additional arguments}
}
\value{
An object of class `sprclmm`
}
\description{
Summary method for PRC LMM model fits
}
\references{
Signorelli, M. (2024). pencal: an R Package for the Dynamic 
Prediction of Survival with Many Longitudinal Predictors. 
The R Journal, 16 (2), 134-153.
}
\seealso{
\code{\link{fit_prclmm}}, \code{\link{print.prclmm}}
}
\author{
Mirko Signorelli
}
