% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print_prclmm.R
\name{print.prclmm}
\alias{print.prclmm}
\title{Print method for PRC LMM model fits}
\usage{
\method{print}{prclmm}(x, digits = 4, ...)
}
\arguments{
\item{x}{an object of class \code{prclmm}}

\item{digits}{number of digits at which the printed estimated regression
coefficients should be rounded (default is 4)}

\item{...}{additional arguments}
}
\value{
Summary information about the fitted PRC-LMM model
}
\description{
Print method for PRC LMM model fits
}
\references{
Signorelli, M. (2024). pencal: an R Package for the Dynamic 
Prediction of Survival with Many Longitudinal Predictors. 
The R Journal, 16 (2), 134-153.
}
\seealso{
\code{\link{fit_prclmm}}, \code{\link{summary.prclmm}}
}
\author{
Mirko Signorelli
}
