% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary_ranefs.R
\name{summary.ranefs}
\alias{summary.ranefs}
\title{Summary for step 2 of PRC}
\usage{
\method{summary}{ranefs}(object, ...)
}
\arguments{
\item{object}{the output of `summarize_lmms` or `summarize_mlpmms`}

\item{...}{additional arguments}
}
\value{
Information about number of predicted random effects and sample size
}
\description{
Summary function to extract basic descriptives from `summarize_lmms`
and `summarize_mlpmms`
}
\references{
Signorelli, M. (2024). pencal: an R Package for the Dynamic 
Prediction of Survival with Many Longitudinal Predictors. 
The R Journal, 16 (2), 134-153.
}
\seealso{
\code{\link{summarize_lmms}}, \code{\link{summarize_mlpmms}}
}
\author{
Mirko Signorelli
}
